  global ConfigWindows
  set n [Unique]
  set win .config$n
  set ConfigWindows [linsert $ConfigWindows 0 $win]

  LinkWindow $head.config $win
  LinkWindow $win.head $head

# catch "destroy $win"
  toplevel $win -screen $display

  SetHelp $win Window

  wm title $win {SimCity Configuration}
  wm iconname $win {SimCity Configuration}
  wm geometry $win 350x480+5+5
  wm positionfrom $win user
  wm sizefrom $win user
  wm maxsize $win 1000 1000
  wm minsize $win 100 100
  wm protocol $win WM_DELETE_WINDOW "DeleteWindow config ConfigWindows $win"

  update idletasks
  return $win
